unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Buttons, ComCtrls, Menus, IniFiles, Registry, MMSystem, Util;

type
  TMainForm = class(TForm)
    PageControl: TPageControl;
    tabPreview: TTabSheet;
    imgPreview: TImage;
    tabColors: TTabSheet;
    lbxColors: TListBox;
    gbxColors: TGroupBox;
    lblRValue: TLabel;
    lblGValue: TLabel;
    lblBValue: TLabel;
    pnlColor: TPanel;
    tabCursor: TTabSheet;
    lbxCursors: TListBox;
    gbxCursors: TGroupBox;
    Label4: TLabel;
    Label1: TLabel;
    txtCursorFile: TEdit;
    pnlCursor: TPanel;
    imgCursor: TImage;
    tabSound: TTabSheet;
    lbxSounds: TListBox;
    gbxSounds: TGroupBox;
    Label2: TLabel;
    txtSoundFile: TEdit;
    pnlPlaySound: TPanel;
    btnPlaySound: TSpeedButton;
    btnStopPlay: TSpeedButton;
    tabFonts: TTabSheet;
    lbxFonts: TListBox;
    gbxFonts: TGroupBox;
    tabDesktop: TTabSheet;
    lbxDesktop: TListBox;
    gbxDesktop: TGroupBox;
    nbkDesktop: TNotebook;
    Label7: TLabel;
    txtDesktopFile: TEdit;
    pnlIcon: TPanel;
    imgIcon: TImage;
    btnPreview: TButton;
    lblHorizontal: TLabel;
    lblVertical: TLabel;
    lblBorderWidth: TLabel;
    lblTitleWrap: TLabel;
    cbxMetricsKind: TComboBox;
    mnuMain: TMainMenu;
    mnuFile: TMenuItem;
    mnuHelp: TMenuItem;
    mnuExit: TMenuItem;
    mnuAbout: TMenuItem;
    N1: TMenuItem;
    mnuOpen: TMenuItem;
    lblFontName: TLabel;
    lblFontSize: TLabel;
    lblFontStyle: TLabel;
    N2: TMenuItem;
    mnuApply: TMenuItem;
    dlgOpen: TOpenDialog;
    mnuCurrent: TMenuItem;
    N3: TMenuItem;
    procedure mnuOpenClick(Sender: TObject);
    procedure lbxColorsDrawItem(Control: TWinControl; index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure lbxCursorsDrawItem(Control: TWinControl; index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure lbxFontsDrawItem(Control: TWinControl; index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure lbxDesktopDrawItem(Control: TWinControl; index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure lbxColorsClick(Sender: TObject);
    procedure lbxCursorsClick(Sender: TObject);
    procedure lbxSoundsClick(Sender: TObject);
    procedure lbxFontsClick(Sender: TObject);
    procedure btnPlaySoundClick(Sender: TObject);
    procedure lbxDesktopClick(Sender: TObject);
    procedure cbxMetricsKindChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure mnuApplyClick(Sender: TObject);
    procedure btnPreviewClick(Sender: TObject);
    procedure mnuCurrentClick(Sender: TObject);
  private
    FThemePath, FThemeName: string;
    
    // ========= Theme Data Start ========== //
    FMyIcons: array[TMyIcon] of string;
    FMyColors: array[TMyColor] of TColor;
    FMyCursors: array[TMyCursor] of string;
    FMySounds: array[TMySound] of string;
    FMyFonts: array[TMyFont] of TLogFont;
    FWallPaper, FScreenSaver: string;
    FWallPaperStyle, FTileWallPaper: Integer;
    FPattern: string;
    
    FIM: TIconMetrics; // ICONMETRICS
    FNM: TNonClientMetrics;
    // ========= Theme Data End ========== //
    
    FMyCursorIcons: array[TMyCursor] of HICON;
    FMyIconIcons: array[TMyIcon] of HICON;
    FWallPaperBits: TBitmap;
    
    procedure ReadThemeFile(const Filename: string);
    procedure LoadDesc;
    procedure LoadThemeComponents;
    
    procedure DrawTheme(DrawCanvas: TCanvas);
    procedure UpdateCaption;
    procedure ApplyTheme;
    procedure LoadCurrentSettings;
  public
    { Public declarations }
  end;
  
var
  MainForm: TMainForm;
  
implementation

uses xUtils, xGraphics, xKernel, xFiles, xFonts, xRegistry, xDesktop, xControls;

{$R *.DFM}

procedure TMainForm.mnuOpenClick(Sender: TObject);
begin
  if dlgOpen.Execute then
  begin
    ReadThemeFile(dlgOpen.Filename);
    UpdateCaption;
  end;
end;

procedure TMainForm.LoadDesc;
var
  J: TMyCursor; 
  K: TMyIcon;   
  L: TMyColor;  
  F: TMyFont;   
  D: TMyDesktop;
  I: TMySound;  
begin
  lbxColors.Items.Clear;
  for L := Low(FMyColors) to High(FMyColors) do
    lbxColors.Items.Add(MyColorDesc[L]);
  
  lbxCursors.Items.Clear;
  for J := Low(FMyCursors) to High(FMyCursors) do
    lbxCursors.Items.Add(MyCursorDesc[J]);
  
  lbxSounds.Items.Clear;
  with TRegIniFile.Create('AppEvents\EventLabels') do
    try
      for I := Low(FMySounds) to High(FMySounds) do
        lbxSounds.Items.Add(ReadString(MySoundDescKey[I], '', MySoundDescKey[I]));
    finally
      Free;
    end;
  
  lbxFonts.Items.Clear;
  for F := Low(FMyFonts) to High(FMyFonts) do
    lbxFonts.Items.Add(MyFontDesc[F]);
  
  lbxDesktop.Items.Clear;
  for K := Low(FMyIcons) to High(FMyIcons) do
    lbxDesktop.Items.Add(MyIconDesc[K]);
  
  for D := Low(TMyDesktop) to High(TMyDesktop) do
    lbxDesktop.Items.Add(MyDesktopDesc[D]);
end;



procedure TMainForm.ReadThemeFile(const Filename: string);
var
  IniFile: TIniFile;
  
  function ReadSetting(const Section: string; Ident: string): string;
  var
    S: string;
  begin
    if Ident = '' then Ident := 'DefaultValue';
    Result := IniFile.ReadString(Section, Ident, '');
    
    S := FThemePath + ExtractFileName(Result);
    
    if FileExistsAfterTruncate(S) then
    begin
      Result := S;
      Exit;
    end;
  end;
  
  procedure ReadOthers;
  var
    S: string;
  begin
    FWallPaper := ReadSetting(secDesktop, 'WallPaper');
    
    FWallPaperStyle := StrToIntDef(ReadSetting(secDesktop, 'WallPaperStyle'), 0);
    FTileWallPaper := StrToIntDef(ReadSetting(secDesktop, 'TileWallPaper'), 0);
    FPattern := ReadSetting(secDesktop, 'Pattern');
    
    S := ReadSetting(secBoot, 'SCRNSAVE.EXE');
    FScreenSaver := S;
  end;
  
  procedure ReadMySounds;
  var
    I: TMySound;
  begin
    for I := Low(FMySounds) to High(FMySounds) do
      FMySounds[I] := ReadSetting('AppEvents\Schemes\Apps\' + MySoundSect[I], '');
  end;
  
  procedure ReadMetrics;
  var
    S: string;
  begin
    S := ReadSetting(secMetrics, 'IconMetrics');
    StringToBuffer(S, @FIM, SizeOf(FIM));
    S := ReadSetting(secMetrics, 'NonClientMetrics');
    StringToBuffer(S, @FNM, SizeOf(FNM));
    
    FMyFonts[mfIcon] := FIM.lfFont;
    FMyFonts[mfCaptionFont] := FNM.lfCaptionFont;
    FMyFonts[mfsmCaptionFont] := FNM.lfSmCaptionFont;
    FMyFonts[mfMenuFont] := FNM.lfMenuFont;
    FMyFonts[mfStatusFont] := FNM.lfStatusFont;
    FMyFonts[mfMessageFont] := FNM.lfMessageFont;
  end;
  
  procedure ReadMyIcons;
  begin
    if IniFile.SectionExists(secMyComputerKey) then
      FMyIcons[mcMyComputer] := ReadSetting(secMyComputerKey, '') // for shit win98's theme
    else
      FMyIcons[mcMyComputer] := ReadSetting(secMyComputer, '');
    
    if IniFile.SectionExists(secNeighborKey) then
      FMyIcons[mcNeighbor] := ReadSetting(secNeighborKey, '')
    else
      FMyIcons[mcNeighbor] := ReadSetting(secNeighbor, '');
    
    if IniFile.SectionExists(secRecycleBinKey) then
    begin
      FMyIcons[mcRecycleBinFull] := ReadSetting(secRecycleBinKey, 'full');
      FMyIcons[mcRecycleBinEmpty] := ReadSetting(secRecycleBinKey, 'empty');
    end else
    begin
      FMyIcons[mcRecycleBinFull] := ReadSetting(secRecycleBin, 'full');
      FMyIcons[mcRecycleBinEmpty] := ReadSetting(secRecycleBin, 'empty');
    end;
  end;
  
  procedure ReadMyColors;
  var
    I: TMyColor;
    S: string;  
  begin
    for I := Low(FMyColors) to High(FMyColors) do
    begin
      S := ReadSetting(secColors, MyColorID[I]);
      if S <> '' then FMyColors[I] := RGBStringToColor(S)
      else
      begin
        if I = GradientActiveTitle then
          FMyColors[I] := FMyColors[ActiveTitle]
        else if I = GradientInActiveTitle then
          FMyColors[I] := FMyColors[InActiveTitle]
        else FMyColors[I] := -1; // w]
      end;
    end;
  end;
  
  procedure ReadMyCursors;
  var
    I: TMyCursor;
  begin
    for I := Low(FMyCursors) to High(FMyCursors) do
      FMyCursors[I] := ReadSetting(secCursors, MyCursorID[I]);
    
    if FMyCursors[mcrArrow] <> '' then
    begin
      Screen.Cursors[200] := LoadCursorFromFile(PChar(FMyCursors[mcrArrow]));
      imgPreview.Cursor := 200;
    end;
  end;
  
begin
  DoBusy(True);
  try
    FThemePath := PathWithSlash(ExtractFilePath(Filename));
    FThemeName := ExtractFileName(Filename);
    
    IniFile := TIniFile.Create(Filename);
    try
      ReadMetrics;
      ReadMyIcons;
      ReadMyColors;
      ReadMyCursors;
      ReadMySounds;
      ReadOthers;
    finally
      IniFile.Free;
    end;
    
    LoadThemeComponents;
    DrawTheme(imgPreview.Canvas);
    
    Invalidate;
  finally
    DoBusy(False);
  end;
end;

procedure TMainForm.UpdateCaption;
begin
  if FThemeName <> '' then
    Caption := Format('%s - %s%s',[Application.Title, FThemePath, FThemeName])
  else
    Caption := Format('%s - ثe]w',[Application.Title]);
end;

procedure TMainForm.DrawTheme(DrawCanvas: TCanvas);
  
  procedure DrawBackgound;
  begin
    // I
    with DrawCanvas do
    begin
      Brush.Style := bsSolid;
      Brush.Color := FMyColors[Background];
      FillRect(ClipRect);
      
      if FileExists(FWallPaper) then
        StretchDraw(ClipRect, FWallPaperBits);
    end;
  end;
  
  procedure DrawIcons;
  const
    IconX         = 75;      
    IconY         = 40;      
    
    offX          = 32 div 2; // icon size @wO 32
    offY: Integer = 40;      
  var
    H       : HICON;  
    I       : TMyIcon;
    wID, Hei: Integer;
    R       : TRect;  
  begin
    with DrawCanvas do
    begin
      Brush.Style := bsSolid;
      Brush.Color := FMyColors[Background];
      Font.Color := CalcForeColor(FMyColors[Background]);
      Font.Handle := CreateFontIndirect(FMyFonts[mfIcon]);
      
      SetTextAlign(Handle, TA_CENTER);
      for I := Low(TMyIcon) to High(TMyIcon) do
      begin
        H := MyLoadIcon(FMyIcons[I], 0);
        DrawIcon(Handle, IconX, IconY + FIM.iVertSpacing * Ord(I), H);
        DestroyIcon(H);
        
        TextOut(IconX + offX, IconY + FIM.iVertSpacing * Ord(I) + offY, MyIconDesc[I]);
        
        wID := TextWidth(MyIconDesc[I]);
        Hei := TextHeight(MyIconDesc[I]);
        R := Rect(IconX + offX, IconY + FIM.iVertSpacing * Ord(I) + offY, IconX + offX, IconY + FIM.iVertSpacing * Ord(I) + offY);
        InflateRect(R, wID div 2, Hei div 2);
      end;
      
      SetTextAlign(Handle, TA_LEFT);
    end;
  end;
  
  procedure DrawWindows;
    
    procedure DrawBorderIcon(ACanvas: TCanvas; ARect: TRect; Kind: Integer);
    var
      W, H, I, OldLeft, OldTop: Integer;
      RT                      : TRect;  
    begin
      RT := ARect;
      Frame3d(ACanvas, ARect, FMyColors[ButtonHilight], FMyColors[ButtonDkShadow], 1);
      Frame3d(ACanvas, ARect, FMyColors[ButtonFace], FMyColors[ButtonShadow], 1);
      ACanvas.Brush.Color := FMyColors[ButtonFace];
      ACanvas.FillRect(ARect);
      
      with RT do
      begin
        W := Right - Left;
        H := Bottom - Top;
        
        // 1: ̤p 2: ̤j 3: 
        case Kind of
          1:
          begin
            Right := Left + W * 63 div 100;
            Left := Left + W * 20 div 100;
            Bottom := Top + H * 85 div 100;
            Top := Top + H * 70 div 100;
            ACanvas.Brush.Color := clBlack;
            ACanvas.FillRect(RT);
          end;
          
          2:
          begin
            OldLeft := Left;
            OldTop := Top;
            
            Right := Left + W * 80 div 100;
            Left := Left + W * 18 div 100;
            Bottom := Top + H * 83 div 100;
            Top := Top + H * 18 div 100;
            ACanvas.Brush.Color := clBlack;
            ACanvas.FillRect(RT);
            
            Left := OldLeft;
            Top := OldTop;
            Right := Left + W * 74 div 100;
            Left := Left + W * 24 div 100;
            Bottom := Top + H * 77 div 100;
            Top := Top + H * 31 div 100;
            ACanvas.Brush.Color := FMyColors[ButtonFace];
            ACanvas.FillRect(RT);
          end;
          
          3:
          begin
            with ACanvas do
            begin
              Pen.Color := clBlack;
              Pen.Width := W div 8;
              I := H * 60 div 100;
              Left := Left + W * 20 div 100;
              Top := Top + H * 16 div 100;
              Right := Left + I;
              Bottom := Top + I;
              
              MoveTo(Left, Top);
              LineTo(Right, Bottom);
              
              MoveTo(Right, Top);
              LineTo(Left, Bottom);
            end;
          end;
        end;
      end;
    end;
    
    procedure DrawScrollButton(ACanvas: TCanvas; ARect: TRect; Kind: Integer);
    var
      W, H: Integer;
      RT  : TRect;  
    begin
      RT := ARect;
      
      Frame3d(ACanvas, RT, FMyColors[ButtonFace], FMyColors[ButtonDkShadow], 1);
      Frame3d(ACanvas, RT, FMyColors[ButtonHilight], FMyColors[ButtonShadow], 1);
      ACanvas.Brush.Color := FMyColors[ButtonFace];
      ACanvas.FillRect(RT);
      
      ACanvas.Brush.Color := FMyColors[ButtonDkShadow];
      W := RT.Right - RT.Left;
      H := RT.Bottom - RT.Top;
      
      case Kind of
        0: ACanvas.Polygon([Point(RT.Left + Round(W * 0.16), RT.Top + Round(H * 0.65)),
          Point(RT.Left + Round(W * 0.74), RT.Top + Round(H * 0.65)),
            Point(RT.Left + Round(W * 0.46), RT.Top + Round(H * 0.25))]);
          
        1: ACanvas.Polygon([Point(RT.Left + Round(W * 0.16), RT.Top + Round(H * 0.25)),
          Point(RT.Left + Round(W * 0.74), RT.Top + Round(H * 0.25)),
            Point(RT.Left + Round(W * 0.46), RT.Top + Round(H * 0.65))]);
      end;
    end;
    
    procedure FillGradientRect(RT: TRect; LeftColor, RightColor: TColor);
    var
      I, W      : Integer;
      R1, g1, b1: Byte;   
      RD, gd, bd: Real;   
    begin
      with DrawCanvas do
      begin
        if LeftColor = RightColor then
        begin
          Brush.Color := LeftColor;
          FillRect(RT);
          Exit;
        end;
        
        W := RT.Right - RT.Left;
        R1 := GetRValue(ColorToRGB(LeftColor));
        g1 := GetGValue(ColorToRGB(LeftColor));
        b1 := GetBValue(ColorToRGB(LeftColor));
        
        RD := (GetRValue(ColorToRGB(RightColor)) - R1) / W;
        gd := (GetGValue(ColorToRGB(RightColor)) - g1) / W;
        bd := (GetBValue(ColorToRGB(RightColor)) - b1) / W;
        
        W := 0;
        for I := RT.Left to RT.Right - 1 do
        begin
          Pen.Color := RGB(R1 + Round(RD * W), g1 + Round(gd * W), b1 + Round(bd * W));
          MoveTo(I, RT.Top);
          LineTo(I, RT.Bottom);
          Inc(W);
        end;
      end;
    end;
    
  var
    RT, oldRT                             : TRect;  
    rtInactive, rtActive, rtMemo, rtMsgBox: TRect;  
    Pos                                   : TPoint; 
    btnSize                               : Integer;
  begin
    with DrawCanvas do
    begin // NONCLIENTMETRICS GetSYSColor
      // eDu@
      RT := Rect(185, 145, 415, 330);
      rtInactive := RT;
      
      // Frame3D |۰ InflateRect...
      Frame3d(DrawCanvas, RT, FMyColors[ButtonFace], FMyColors[ButtonDkShadow], 1);
      Frame3d(DrawCanvas, RT, FMyColors[ButtonHilight], FMyColors[ButtonShadow], 1);
      Frame3d(DrawCanvas, RT, FMyColors[InactiveBorder], FMyColors[InactiveBorder], FNM.iBorderWidth);
      
      // border
      Brush.Color := FMyColors[ButtonFace];
      FillRect(RT);
      
      // caption
      InflateRect(RT, - 1, - 1);
      //    FNM.iCaptionwidth := 30;
      //    FNM.iCaptionheight := 30;
      RT.Bottom := RT.Top + FNM.iCaptionHeight;
      FillGradientRect(RT, FMyColors[InActiveTitle], FMyColors[GradientInActiveTitle]);
      
      Brush.Style := bsClear;
      Font.Handle := CreateFontIndirect(FMyFonts[mfCaptionFont]);
      Font.Color := FMyColors[InactiveTitleText];
      
      Pos.X := RT.Left + 2; // LOGFONT
      Pos.Y := RT.Top + (RT.Bottom - RT.Top - TextHeight(sInactiveWindow)) div 2;
      TextRect(RT, Pos.X, Pos.Y, sInactiveWindow);
      oldRT := RT; //  RT d_ӫݷ|n
      Brush.Style := bsSolid;
      
      // eees
      btnSize := FNM.iCaptionWidth - 2;
      InflateRect(RT, - 2, - 2);
      
      RT.Left := RT.Right - btnSize;
      DrawBorderIcon(DrawCanvas, RT, 3); // 
      
      OffsetRect(RT, - (btnSize + 1), 0);
      DrawBorderIcon(DrawCanvas, RT, 2); // ̤j
      
      OffsetRect(RT, - btnSize, 0);
      DrawBorderIcon(DrawCanvas, RT, 1); // ̤p
      
      // e Active Window
      // oɭ oldRT ODϥΤ Title
      RT := oldRT;
      RT.Left := RT.Left + 1;
      RT.Top := RT.Bottom + 1;
      RT.Right := rtInactive.Right + btnSize + 6 + FNM.iBorderWidth;
      RT.Bottom := rtInactive.Bottom + 4;
      rtActive := RT;
      
      // Frame3D |۰ InflateRect...
      Frame3d(DrawCanvas, RT, FMyColors[ButtonFace], FMyColors[ButtonDkShadow], 1);
      Frame3d(DrawCanvas, RT, FMyColors[ButtonHilight], FMyColors[ButtonShadow], 1);
      Frame3d(DrawCanvas, RT, FMyColors[activeBorder], FMyColors[activeBorder], FNM.iBorderWidth);
      
      // border
      Brush.Color := FMyColors[ButtonFace];
      FillRect(RT);
      
      // caption
      InflateRect(RT, - 1, - 1);
      RT.Bottom := RT.Top + FNM.iCaptionHeight;
      
      FillGradientRect(RT, FMyColors[ActiveTitle], FMyColors[GradientActiveTitle]);
      
      Brush.Style := bsClear;
      Font.Color := FMyColors[TitleText];
      Pos.X := RT.Left + 2;
      Pos.Y := RT.Top + (RT.Bottom - RT.Top - TextHeight(sActiveWindow)) div 2;
      TextRect(RT, Pos.X, Pos.Y, sActiveWindow);
      Brush.Style := bsSolid;
      
      // eees
      btnSize := FNM.iCaptionWidth - 2;
      InflateRect(RT, - 2, - 2);
      
      oldRT := RT; //  RT d_ӫݷ|n
      RT.Left := RT.Right - btnSize;
      DrawBorderIcon(DrawCanvas, RT, 3); // 
      
      OffsetRect(RT, - (btnSize + 1), 0);
      DrawBorderIcon(DrawCanvas, RT, 2); // ̤j
      
      OffsetRect(RT, - btnSize, 0);
      DrawBorderIcon(DrawCanvas, RT, 1); // ̤p
      
      // NONCLIENTMETRICS
      // oɭ oldRT OϥΤ Title
      RT := oldRT;
      OffsetRect(RT, 0, FNM.iCaptionHeight);
      RT.Bottom := RT.Top + FNM.iMenuHeight;
      oldRT := RT;
      
      // \
      Brush.Color := FMyColors[MenuColor];
      FillRect(RT);
      Font.Handle := CreateFontIndirect(FMyFonts[mfMenuFont]);
      Font.Color := FMyColors[MenuText];
      
      RT.Top := RT.Top - 1;
      Pos.X := RT.Left + 4; // LOGFONT
      Pos.Y := RT.Top + (RT.Bottom - RT.Top - TextHeight(sNormal + sDisabled + sSelected)) div 2;
      //    TextOut(Pos.X, Pos.Y, 'з');
      TextOut(Pos.X, Pos.Y, sNormal);
      
      Pos.X := RT.Left + TextWidth(sNormal + '12');
      
      Font.Color := FMyColors[ButtonHilight];
      TextOut(Pos.X + 1, Pos.Y + 1, sDisabled); // kU
      Font.Color := FMyColors[GrayText];
      //    TextOut(Pos.X, Pos.Y, '');
      Brush.Style := bsClear;
      TextOut(Pos.X, Pos.Y, sDisabled);
      Brush.Style := bsSolid;
      
      Pos.X := Pos.X + TextWidth(sDisabled + '123');
      Brush.Color := FMyColors[Hilight];
      Font.Color := FMyColors[HilightText];
      RT.Left := Pos.X - TextWidth('1');
      RT.Right := Pos.X + TextWidth(sSelected) + TextWidth('1');
      //    TextRect(RT, Pos.X, Pos.Y, '');
      RT.Bottom := RT.Bottom - 1;
      TextRect(RT, Pos.X, Pos.Y, sSelected);
      
      // o oldRt V Menu
      RT := oldRT;
      OffsetRect(RT, 0, FNM.iMenuHeight);
      InflateRect(RT, 2, 2);
      RT.Bottom := rtActive.Bottom - 2 - FNM.iBorderWidth;
      rtMemo := RT;
      
      // memo
      Frame3d(DrawCanvas, RT, FMyColors[ButtonShadow], FMyColors[ButtonHilight], 1);
      Frame3d(DrawCanvas, RT, FMyColors[ButtonDkShadow], FMyColors[ButtonFace], 1);
      Brush.Color := FMyColors[Window];
      FillRect(RT);
      
      Font.name := 'System';
      Font.Size := 12;
      Font.Color := clBlack;
      //    TextOut(RT.Left + 2, RT.Top + 2, 'r');
      TextOut(RT.Left + 2, RT.Top + 2, sWindowText);
      
      // scroll box
      RT.Left := RT.Right - FNM.iScrollWidth;
      Brush.Color := FMyColors[ScrollBar];
      FillRect(RT);
      
      RT.Bottom := RT.Top + FNM.iScrollHeight;
      DrawScrollButton(DrawCanvas, RT, 0); // VW
      
      RT.Bottom := rtMemo.Bottom;
      RT.Top := RT.Bottom - FNM.iScrollHeight;
      DrawScrollButton(DrawCanvas, RT, 1); // VU
      // }le message box
      RT.Top := rtMemo.Top + (rtMemo.Bottom - rtMemo.Top) * 1 div 3;
      RT.Left := rtMemo.Left + 3;
      RT.Right := rtMemo.Left + (rtMemo.Right - rtMemo.Left) * 3 div 4;
      RT.Bottom := ClipRect.Bottom - 20; // messagebox ݶZe 20 points
      
      // Frame3D |۰ InflateRect...
      rtMsgBox := RT;
      Frame3d(DrawCanvas, RT, FMyColors[ButtonFace], FMyColors[ButtonDkShadow], 1);
      Frame3d(DrawCanvas, RT, FMyColors[ButtonHilight], FMyColors[ButtonShadow], 1);
      
      // border
      Brush.Color := FMyColors[ButtonFace];
      FillRect(RT);
      
      // caption
      InflateRect(RT, - 1, - 1);
      RT.Bottom := RT.Top + FNM.iCaptionHeight;
      oldRT := RT;
      Brush.Color := FMyColors[ActiveTitle];
      
      
      Font.Handle := CreateFontIndirect(FMyFonts[mfCaptionFont]);
      Font.Color := FMyColors[TitleText];
      Pos.X := RT.Left + 2; // LOGFONT
      Pos.Y := RT.Top + Round((RT.Bottom - RT.Top - TextHeight(sMessageBox)) / 2);
      //    TextRect(RT, Pos.X, Pos.Y, 'T');
      TextRect(RT, Pos.X, Pos.Y, sMessageBox);
      
      // eees
      btnSize := FNM.iCaptionWidth - 2;
      InflateRect(RT, - 2, - 2);
      
      RT.Left := RT.Right - btnSize;
      DrawBorderIcon(DrawCanvas, RT, 3); // 
      
      // NONCLIENTMETRICS
      // e message box Tr
      Font.Handle := CreateFontIndirect(FMyFonts[mfMessageFont]);
      Font.Color := FMyColors[WindowTextColor];
      
      RT := oldRT;
      Pos.X := RT.Left + 4;
      Pos.Y := RT.Top + FNM.iCaptionHeight + 4;
      TextOut(Pos.X, Pos.Y, sMessageText);
      RT.Left := Pos.X;
      RT.Top := Pos.Y;
      RT.Right := Pos.X + TextWidth(sMessageText);
      RT.Bottom := Pos.Y + TextHeight(sMessageText);
      
      Font.Handle := CreateFontIndirect(FMyFonts[mfCaptionFont]);
      Font.Color := FMyColors[ButtonText];
      Pos.X := TextWidth(sOK + '123456');
      Pos.Y := TextHeight(sOK) * 3 div 2;
      
      // MessageBox  button
      RT.Left := rtMsgBox.Left + ((rtMsgBox.Right - rtMsgBox.Left) - Pos.X) div 2;
      RT.Right := RT.Left + Pos.X;
      RT.Bottom := rtMsgBox.Bottom - 10;
      RT.Top := RT.Bottom - Pos.Y;
      
      Frame3d(DrawCanvas, RT, FMyColors[ButtonHilight], FMyColors[ButtonDkShadow], 1);
      Frame3d(DrawCanvas, RT, FMyColors[ButtonFace], FMyColors[ButtonShadow], 1);
      Brush.Color := FMyColors[ButtonFace];
      
      Pos.X := RT.Left + (Pos.X - TextWidth(sOK)) div 2 - 2; // LOGFONT
      Pos.Y := RT.Top + (RT.Bottom - RT.Top - TextHeight(sOK)) div 2;
      TextRect(RT, Pos.X, Pos.Y, sOK);
      
      // NONCLIENTMETRICS
      // Hint eb Button kU
      // ثe RT O button
      Brush.Color := FMyColors[InfoWindow];
      Font.Color := FMyColors[InfoText];
      Font.Handle := CreateFontIndirect(FMyFonts[mfMessageFont]);
      
      RT.Left := (RT.Left + RT.Right) div 2;
      RT.Top := RT.Bottom + 5;
      RT.Right := RT.Left + TextWidth(sHint) + TextWidth('12');
      RT.Bottom := RT.Top + TextHeight(sHint);
      TextRect(RT, RT.Left + TextWidth('1'), RT.Top, sHint);
    end;
  end;
  
begin
  DoBusy(True);
  try
    DrawBackgound;
    DrawIcons;
    DrawWindows;
  finally
    DoBusy(False);
  end;
end;

procedure TMainForm.ApplyTheme;
  
  procedure SetMetrics;
  begin
    FIM.cbSize := SizeOf(FIM);
    SystemParametersInfo(SPI_SETICONMETRICS, SizeOf(FIM), @FIM, SPIF_UPDATEINIFILE or SPIF_SENDWININICHANGE);
    
    FNM.cbSize := SizeOf(FNM);
    SystemParametersInfo(SPI_SETNONCLIENTMETRICS, SizeOf(FNM), @FNM, SPIF_UPDATEINIFILE or SPIF_SENDWININICHANGE);
  end;
  
  procedure SetColors;
  var
    K          : TMyColor;               
    CC         : Integer;                 // count of colors
    SaveColorNo: array[0..40] of Integer;
    SaveColors : array[0..40] of TColor; 
  begin
    CC := 0;
    for K := Low(TMyColor) to High(TMyColor) do
    begin
      SaveColorNo[CC] := MyColorNo[K];
      SaveColors[CC] := FMyColors[K];
      Inc(CC);
    end;
    
    // ]wC
    SetSysColors(CC, SaveColorNo, SaveColors);
    
    // nxsb registry
    with TRegIniFile.Create('Control Panel') do
      try
        for K := Low(TMyColor) to High(TMyColor) do
          WriteString('Colors', MyColorID[K], ColorToRGBString(FMyColors[K]));
      finally
        Free;
      end;
  end;
  
var
  I       : TMySound;             
  J       : TMyCursor;            
  hc      : HCURSOR;              
  Buf     : array[0..255] of Char;
  Registry: TRegistry;            
begin
  DoBusy(True);
  try
    with TRegIniFile.Create('Control Panel') do
      try
        WriteString('Desktop', 'WallPaperStyle', IntToStr(FWallPaperStyle));
        WriteString('Desktop', 'TileWallPaper', IntToStr(FTileWallPaper));
        
        if FWallPaper = '' then
          SystemParametersInfo(SPI_SETDESKWALLPAPER, 0, StrPCopy(@Buf, ''), SPIF_UPDATEINIFILE or SPIF_SENDWININICHANGE)
        else if (ExtractFileExt(AnsiUpperCase(FWallPaper)) = '.BMP') then
          SystemParametersInfo(SPI_SETDESKWALLPAPER, 0, StrPCopy(@Buf, FWallPaper), SPIF_UPDATEINIFILE or SPIF_SENDWININICHANGE)
        else
        begin
          FWallPaperBits.SaveToFile(GetMyWallPaperFile);
          SystemParametersInfo(SPI_SETDESKWALLPAPER, 0, PChar(GetMyWallPaperFile), SPIF_UPDATEINIFILE or SPIF_SENDWININICHANGE);
        end;
        
        SystemParametersInfo(SPI_SETDESKPATTERN, 0, StrPCopy(@Buf, FPattern), SPIF_UPDATEINIFILE or SPIF_SENDWININICHANGE);
        WriteString('Desktop', 'SCRNSAVE.EXE', FScreenSaver);
        
        // N cursors gJ registry
        for J := Low(FMyCursors) to High(FMyCursors) do
          WriteString('Cursors', MyCursorID[J], FMyCursors[J]);
      finally
        Free;
      end;
    
    Registry := TRegistry.Create;
    with Registry do
      try
        // gJ Sounds
        for I := Low(FMySounds) to High(FMySounds) do
        begin
          if OpenKey('AppEvents\Schemes\Apps\' + MySoundSect[I], True) then
          begin
            WriteString('', FMySounds[I]);
            CloseKey;
          end;
        end;
        
        // gJ Icon
        RootKey := HKEY_CLASSES_ROOT;
        WriteKeyDefaultValue(Registry, secMyComputer, TruncateTrailIfNotDLL(FMyIcons[mcMyComputer]));
        WriteKeyDefaultValue(Registry, secNeighbor, TruncateTrailIfNotDLL(FMyIcons[mcNeighbor]));
        
        if OpenKey(secRecycleBin, True) then
        begin
          WriteString('Full', TruncateTrailIfNotDLL(FMyIcons[mcRecycleBinFull]));
          WriteString('Empty', TruncateTrailIfNotDLL(FMyIcons[mcRecycleBinEmpty]));
          WriteString('', TruncateTrailIfNotDLL(FMyIcons[mcRecycleBinEmpty]));
          
          CloseKey;
        end;
      finally
        Free;
      end;
    
    //  cursors
    for J := Low(FMyCursors) to High(FMyCursors) do
    begin
      // win95/98  animated cursor  setsystemcursor D, |
      if (AnsiUpperCase(ExtractFileExt(FMyCursors[J])) = '.ANI') and (Win32Platform <> VER_PLATFORM_WIN32_NT) then continue;
      
      hc := MyLoadCursor(J, FMyCursors[J]);
      if hc <> 0 then SetSystemCursor(hc, mycursorno[J]);
    end;
    
    SetColors;
    SetMetrics;
    
    // jୱϥܧs
    RebuildIconCache;
  finally
    DoBusy(False);
  end;
end;

procedure TMainForm.lbxColorsDrawItem(Control: TWinControl; index: Integer;
    Rect: TRect; State: TOwnerDrawState);
begin
  with lbxColors do
  begin
    Canvas.FillRect(Rect);
    DrawText(Canvas.Handle, PChar(Items[index]), Length(Items[index]), Rect, DT_VCENTER or DT_SINGLELINE);
    
    if FMyColors[TMyColor(index)] <> -1 then
    begin
      Canvas.Brush.Style := bsSolid;
      Canvas.Brush.Color := FMyColors[TMyColor(index)];
      Rect.Left := Rect.Right - (Rect.Bottom - Rect.Top);
      Canvas.FillRect(Rect);
    end;
  end;
end;

procedure TMainForm.lbxCursorsDrawItem(Control: TWinControl;
    index: Integer; Rect: TRect; State: TOwnerDrawState);
begin
  with lbxCursors do
  begin
    Canvas.FillRect(Rect);
    DrawText(Canvas.Handle, PChar(Items[index]), Length(Items[index]), Rect, DT_VCENTER or DT_SINGLELINE);
    Windows.DrawIcon(Canvas.Handle, Rect.Right - 40, Rect.Top, FMyCursorIcons[TMyCursor(index)]);
  end;
end;

procedure TMainForm.lbxFontsDrawItem(Control: TWinControl; index: Integer;
    Rect: TRect; State: TOwnerDrawState);
var
  H, L: Integer;
  S   : string; 
begin
  with lbxFonts.Canvas do
  begin
    FillRect(Rect);
    DrawText(Handle, PChar(lbxFonts.Items[index]), Length(lbxFonts.Items[index]), Rect, DT_VCENTER or DT_SINGLELINE);
    
    Font.Handle := CreateFontIndirect(FMyFonts[TMyFont(index)]);
    S := fonttostring(Font, False);
    H := Rect.Bottom - Rect.Top;
    L := Rect.Right - TextWidth(S) - 4;
    if L < 150 then L := 150;
    TextOut(L, Rect.Top + (H - TextHeight(S)) div 2, S);
  end;
end;

procedure TMainForm.lbxDesktopDrawItem(Control: TWinControl;
    index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  R: TRect;
begin
  with lbxDesktop do
  begin
    Canvas.FillRect(Rect);
    DrawText(Canvas.Handle, PChar(Items[index]), Length(Items[index]), Rect, DT_VCENTER or DT_SINGLELINE);
    
    case index of
      0..3: Windows.DrawIcon(Canvas.Handle, Rect.Right - 40, Rect.Top, FMyIconIcons[TMyIcon(index)]);
        
      4: // wallpaper
      begin
        R := Rect;
        R.Right := R.Right - 8;
        R.Left := R.Right - 32;
        R.Bottom := R.Top + 32;
        Canvas.StretchDraw(R, FWallPaperBits);
      end;
    end;
  end;
end;

procedure TMainForm.lbxColorsClick(Sender: TObject);
begin
  if FMyColors[TMyColor(lbxColors.ItemIndex)] <> -1 then
  begin
    pnlColor.Color := FMyColors[TMyColor(lbxColors.ItemIndex)];
    lblRValue.Caption := Format('j: %d',[GetRValue(pnlColor.Color)]);
    lblGValue.Caption := Format('j: %d',[GetGValue(pnlColor.Color)]);
    lblBValue.Caption := Format('Ŧj: %d',[GetBValue(pnlColor.Color)]);
  end else
  begin
    pnlColor.Color := clBtnFace;
    lblRValue.Caption := '';
    lblGValue.Caption := '';
    lblBValue.Caption := '';
  end;
end;

procedure TMainForm.lbxCursorsClick(Sender: TObject);
begin
  with imgCursor.Picture.Icon do
  begin
    ReleaseHandle;
    InvalidateRect(pnlCursor.Handle, nil, True);
    Handle := FMyCursorIcons[TMyCursor(lbxCursors.ItemIndex)];
  end;
  
  txtCursorFile.Text := FMyCursors[TMyCursor(lbxCursors.ItemIndex)];
end;

procedure TMainForm.lbxSoundsClick(Sender: TObject);
begin
  txtSoundFile.Text := FMySounds[TMySound(lbxSounds.ItemIndex)];
  EnableControl(pnlPlaySound, (auxGetNumDevs <> 0) and (FileExists(FMySounds[TMySound(lbxSounds.ItemIndex)])));
end;

procedure TMainForm.lbxFontsClick(Sender: TObject);
var
  F: TLogFont; // logfont
  S: string;  
begin
  F := FMyFonts[TMyFont(lbxFonts.ItemIndex)];
  lblFontName.Caption := Format('W: %s',[F.lfFaceName]);
  lblFontSize.Caption := Format('jp: %d',[ - MulDiv(F.lfHeight, 72, Screen.PixelsPerInch)]);
  
  S := '˦: ';
  if F.lfWeight >= 700 then S := S + '';
  if F.lfItalic <> 0 then S := S + '';
  if F.lfUnderline <> 0 then S := S + 'u';
  lblFontStyle.Caption := S;
end;

procedure TMainForm.btnPlaySoundClick(Sender: TObject);
begin
  case (Sender as TComponent).Tag of
    0: sndPlaySound(PChar(txtSoundFile.Text), SND_ASYNC);
    1: sndPlaySound(nil, SND_ASYNC);
  end;
end;

procedure TMainForm.lbxDesktopClick(Sender: TObject);
type
  TPatternArray = array[0..7, 0..7] of Boolean;
  
  function ParsePatternString(sPattern: string; var aPattern: TPatternArray): Boolean;
  var
    X, Y, I, Value: Integer;
  begin
    Y := 0;
    while Length(sPattern) > 0 do
    begin
      I := Pos(' ', sPattern);
      if I <> 0 then
      begin
        Value := StrToIntDef(Copy(sPattern, 1, I - 1), 0);
        Delete(sPattern, 1, I);
      end else
      begin
        Value := StrToIntDef(sPattern, 0);
        sPattern := '';
      end;
      
      X := 7;
      while X >= 0 do
      begin
        aPattern[Y, X] := Boolean(Value and 1);
        Value := Value shr 1;
        dec(X);
      end;
      
      Inc(Y);
    end;
    
    Result := Y > 7; // successfully parsed
  end;
  
  procedure ShowPattern;
  const
    { ӬO '(None)',  Windows NT 4.0 c餤媩N½
     '(L)', uOFI }
    NON_PATTERN_STR = '(None)';
  var
    aPattern  : TPatternArray;
    X, Y, W, H: Integer;      
  begin
    { draw each cell base on aPattern boolean array }
    with imgIcon do
    begin
      { calculate the width of each cell and adjust total width }
      
      Picture.Bitmap.Width := ClientWidth;
      Picture.Bitmap.Height := ClientHeight;
      
      W := ClientWidth div 8;
      H := ClientHeight div 8;
      
      with Picture.Bitmap.Canvas do
      begin
        Brush.Style := bsSolid;
        Brush.Color := FMyColors[Background];
        { first fill all the canvas with clBackground color
        (corresponding COLOR_BACKGROUND in WINDOWS.PAS) }
        FillRect(ClipRect);
        
        if (FPattern = '') or (FPattern = NON_PATTERN_STR) or (FPattern = '(L)') then Exit;
        
        { decomposite the pattern string }
        if not ParsePatternString(FPattern, aPattern) then Exit;
        
        Brush.Color := CalcForeColor(FMyColors[Background]);
        
        for Y := 0 to 7 do
          for X := 0 to 7 do
            if aPattern[Y, X] then FillRect(Rect(X * W, Y * H, (X + 1) * W, (Y + 1) * H));
      end;
    end;
  end;
  
var
  R: TRect;
begin
  imgIcon.Picture.Icon.ReleaseHandle;
  InvalidateRect(pnlIcon.Handle, nil, True);
  SendMessage(GetWindow(pnlIcon.Handle, GW_CHILD), WM_CLOSE, 0, 0);
  btnPreview.Enabled := False;
  
  if lbxDesktop.ItemIndex < 7 then nbkDesktop.PageIndex := 0;
  with lbxDesktop do
  begin
    case ItemIndex of
      0..3: // desktop icon
      begin
        txtDesktopFile.Text := TruncateTrailIfNotDLL(FMyIcons[TMyIcon(ItemIndex)]);
        imgIcon.Picture.Icon.Handle := FMyIconIcons[TMyIcon(ItemIndex)];
      end;
      
      4:
      begin // Wallpaper
        txtDesktopFile.Text := FWallPaper;
        
        if (FWallPaper <> '') then // valid wallpaper
          
          with imgIcon do
          begin
            R := Rect(0, 0, ClientWidth, ClientHeight);
            Picture.Bitmap.Width := ClientWidth;
            Picture.Bitmap.Height := ClientHeight;
            Canvas.StretchDraw(R, FWallPaperBits);
          end;
      end;
      
      5:
      begin // pattern
        txtDesktopFile.Text := '';
        
        ShowPattern;
      end;
      
      6:
      begin // screen saver
        txtDesktopFile.Text := FScreenSaver;
        
        if FScreenSaver <> '' then
        begin
          WinExec(PChar(Format('%s -p %d',[FScreenSaver, pnlIcon.Handle])), SW_SHOW);
          btnPreview.Enabled := True;
        end;
      end;
      
      7:
      begin // window metrics
        nbkDesktop.PageIndex := 1;
        Application.ProcessMessages;
        
        lblBorderWidth.Caption := Format('ؼe: %d',[FNM.iBorderWidth]);
        lblTitleWrap.Caption := Format('ϥܤr: %d',[FIM.iTitleWrap]);
        cbxMetricsKind.ItemIndex := 0;
        cbxMetricsKindChange(cbxMetricsKind);
      end;
      
    else txtDesktopFile.Text := '';
    end;
    
  end;
end;

procedure TMainForm.cbxMetricsKindChange(Sender: TObject);
var
  H, V: Integer;
begin
  case cbxMetricsKind.ItemIndex of
    0:
    begin
      H := FNM.iMenuWidth;
      V := FNM.iMenuHeight;
    end;
    
    1:
    begin
      H := FNM.iCaptionWidth;
      V := FNM.iCaptionHeight;
    end;
    
    2:
    begin
      H := FNM.iScrollWidth;
      V := FNM.iScrollHeight;
    end;
    
    3:
    begin
      H := FIM.iHorzSpacing;
      V := FIM.iVertSpacing;
    end;
    
    4:
    begin
      H := FNM.iSmCaptionWidth;
      V := FNM.iSmCaptionHeight;
    end;
  end;
  
  lblHorizontal.Caption := Format(': %d',[H]);
  lblVertical.Caption := Format(': %d',[V]);
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  PageControl.ActivePage := tabPreview;
  LoadDesc;
  
  FWallPaperBits := TBitmap.Create;
  
  LoadCurrentSettings;
  UpdateCaption;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  FWallPaperBits.Free;
end;

procedure TMainForm.mnuApplyClick(Sender: TObject);
begin
  if FThemeName = '' then Exit;
  
  ApplyTheme;
end;

procedure TMainForm.btnPreviewClick(Sender: TObject);
begin
  if FScreenSaver <> '' then
    WinExec(PChar(Format('%s -s',[FScreenSaver])), SW_SHOW);
end;

procedure TMainForm.LoadCurrentSettings;
var
  I                 : TMySound; 
  K                 : TMyColor; 
  J                 : TMyCursor;
  Registry          : TRegistry;
  bLocalDesktopIcons: Boolean;  
begin
  FThemeName := '';
  
  FillChar(FIM, 0, SizeOf(FIM));
  FIM.cbSize := SizeOf(FIM);
  SystemParametersInfo(SPI_GETICONMETRICS, SizeOf(FIM), @FIM, 0);
  FillChar(FNM, 0, SizeOf(FNM));
  FNM.cbSize := SizeOf(FNM);
  SystemParametersInfo(SPI_GETNONCLIENTMETRICS, SizeOf(FNM), @FNM, 0);
  
  FMyFonts[mfIcon] := FIM.lfFont;
  FMyFonts[mfCaptionFont] := FNM.lfCaptionFont;
  FMyFonts[mfsmCaptionFont] := FNM.lfSmCaptionFont;
  FMyFonts[mfMenuFont] := FNM.lfMenuFont;
  FMyFonts[mfStatusFont] := FNM.lfStatusFont;
  FMyFonts[mfMessageFont] := FNM.lfMessageFont;
  
  Registry := TRegistry.Create;
  try
    FMyIcons[mcMyComputer] := '';
    FMyIcons[mcNeighbor] := '';
    FMyIcons[mcRecycleBinFull] := '';
    FMyIcons[mcRecycleBinEmpty] := '';
    
    for bLocalDesktopIcons := False to True do
    begin
      if bLocalDesktopIcons then
        Registry.RootKey := HKEY_LOCAL_MACHINE
      else
        Registry.RootKey := HKEY_CURRENT_USER;
      
      // desktop icons ..
      if FMyIcons[mcMyComputer] = '' then
        FMyIcons[mcMyComputer] := ReadKeyDefaultValue(Registry, secMyComputerKey);
      if FMyIcons[mcNeighbor] = '' then
        FMyIcons[mcNeighbor] := ReadKeyDefaultValue(Registry, secNeighborKey);
      
      with Registry do
        if OpenKey(secRecycleBinKey, False) then
        begin
          if FMyIcons[mcRecycleBinFull] = '' then
            FMyIcons[mcRecycleBinFull] := ReadString('Full');
          if FMyIcons[mcRecycleBinEmpty] = '' then
            FMyIcons[mcRecycleBinEmpty] := ReadString('Empty');
          CloseKey;
        end;
    end;
    
    Registry.RootKey := HKEY_CURRENT_USER;
    // sounds
    for I := Low(FMySounds) to High(FMySounds) do
      FMySounds[I] := ReadKeyDefaultValue(Registry, 'AppEvents\Schemes\Apps\' + MySoundSect[I]);
  finally
    Registry.Free;
  end;
  
  with TRegIniFile.Create('Control Panel') do
    try
      // others
      FWallPaperStyle := StrToIntDef(ReadString('Desktop', 'WallPaperStyle', '0'), 0);
      FTileWallPaper := StrToIntDef(ReadString('Desktop', 'TitleWallPaper', '0'), 0);
      
      FWallPaper := ReadString('Desktop', 'WallPaper', '');
      
      FScreenSaver := ReadString('Desktop', 'SCRNSAVE.EXE', '');
      FPattern := ReadString('Desktop', 'Pattern', '');
      
      //  cursors
      for J := Low(FMyCursors) to High(FMyCursors) do
        FMyCursors[J] := ReadString('Cursors', MyCursorID[J], '');
    finally
      Free;
    end;
  
  // Colors
  for K := Low(TMyColor) to High(TMyColor) do
    FMyColors[K] := GetSysColor(MyColorNo[K]);
  
  LoadThemeComponents;
  DrawTheme(imgPreview.Canvas);
  
  Invalidate;
end;

procedure TMainForm.LoadThemeComponents;
var
  I: TMyCursor;
  K: TMyIcon;  
begin
  for I := Low(FMyCursors) to High(FMyCursors) do
    FMyCursorIcons[I] := CursorToIcon(MyLoadCursor(I, FMyCursors[I]), True);
  
  for K := Low(FMyIcons) to High(FMyIcons) do
    FMyIconIcons[K] := MyLoadIcon(FMyIcons[K], 0);
  
  if FileExists(FWallPaper) then
    FWallPaperBits.Handle := NViewLibLoad(PChar(FWallPaper), False)
  else
    FWallPaperBits.ReleaseHandle;
end;

procedure TMainForm.mnuCurrentClick(Sender: TObject);
begin
  LoadCurrentSettings;
  UpdateCaption;
end;

end.

