unit util;

interface

uses Windows, Classes, SysUtils, Graphics, Forms;

const
  TILE_WIDTH      = 32; // ϤeI
  TILE_HEIGHT     = 32; // ϤI

  TILE_NUM_X      = 10; // eb
  TILE_NUM_Y      = 10; // eab

  FN_TERR_ARCHIVE = 'TERR.TIA'; // aιϮw
  FN_ITEM_ARCHIVE = 'ITEM.TIA'; // ~Ϯw
  FN_ROLEBITS     = 'ROLEBITS.BMP'; // 

  FN_MAP_PREFIX   = 'MAP'; // dɦW (MAP??.DAT)
  FN_MAP_EXT      = '.DAT'; // dɰɦW

  SIG_MYFILE      = 'Xshadow_Stock'; // ϮwΦaɮתY

procedure CheckSignature(fs: TFileStream; const Sig: string);

function IsWindows98: Boolean;
procedure TransparentBlt(DestDC: HDC; srcBmp: THandle; DestX, DestY: Integer; TransColor: Longint);

implementation

procedure CheckSignature(fs: TFileStream; const Sig: string);
var
  S: string;
begin
  SetLength(S, Length(Sig));
  fs.read (S[1], Length(Sig));
  if CompareText(S, Sig) <> 0 then
    raise Exception.Create('File signature not match');
end;

function IsWindows98: Boolean;
begin
  Result := (Win32Platform = VER_PLATFORM_WIN32_WINDOWS) and
    ((Win32MajorVersion > 4) or
    ((Win32MajorVersion = 4) and (Win32MinorVersion >= 10)));
end;

procedure TransparentBlt(DestDC: HDC; srcBmp: THandle; DestX, DestY: Integer; TransColor: Longint);
var
  bmp         : Windows.TBitmap; {description of the source bitmap}
  OrigColor   : Longint;         {Holds original background color from source DC}
  bSuccess    : Boolean;         {Stores result of call to Windows API}

  SrcDC       : HDC;             {source bitmap (color)}
  SaveDC      : HDC;             {backup copy of source bitmap}
  MaskDC      : HDC;             {mask bitmap (monochrome)}
  invDC       : HDC;             {inverse of mask bitmap (monochrome)}
  resultDC    : HDC;             {combination of source bitmap & background}

  hResultBmp  : THandle;         {Bitmap combination of source & background}
  hSaveBmp    : THandle;         {Bitmap stores backup copy of source bitmap}
  hMaskBmp    : THandle;         {Bitmap stores mask (monochrome)}
  hInvBmp     : THandle;         {Bitmap holds inverse of mask (monochrome)}
  hPrevBmp    : THandle;         {Bitmap holds previous bitmap selected in DC}
  hSrcPrevBmp : THandle;         {Holds previous bitmap in source DC}
  hSavePrevBmp: THandle;         {Holds previous bitmap in saved DC}
  hDestPrevBmp: THandle;         {Holds previous bitmap in destination DC}
  hMaskPrevBmp: THandle;         {Holds previous bitmap in the mask DC}
  hInvPrevBmp : THandle;         {Holds previous bitmap in inverted mask DC}
begin
  {Retrieve bitmap to get width (bmp.bmWidth) & height (bmp.bmHeight)}
  bSuccess := Boolean(GetObject(srcBmp, SizeOf(bmp), @bmp));
  SrcDC := CreateCompatibleDC(DestDC); {Create DC to hold stage}
  SaveDC := CreateCompatibleDC(DestDC); {Create DC to hold stage}
  MaskDC := CreateCompatibleDC(DestDC); {Create DC to hold stage}
  invDC := CreateCompatibleDC(DestDC); {Create DC to hold stage}
  resultDC := CreateCompatibleDC(DestDC); {Create DC to hold stage}

  {Create monochrome bitmaps for the mask-related bitmaps:}
  hMaskBmp := CreateBitmap(bmp.bmWidth, bmp.bmHeight, 1, 1, nil);
  hInvBmp := CreateBitmap(bmp.bmWidth, bmp.bmHeight, 1, 1, nil);

  {Create color bitmaps for final result & stored copy of source}
  hResultBmp := CreateCompatibleBitmap(DestDC, bmp.bmWidth, bmp.bmHeight);
  hSaveBmp := CreateCompatibleBitmap(DestDC, bmp.bmWidth, bmp.bmHeight);
  hSrcPrevBmp := SelectObject(SrcDC, srcBmp); {Select bitmap in DC}
  hSavePrevBmp := SelectObject(SaveDC, hSaveBmp); {Select bitmap in DC}
  hMaskPrevBmp := SelectObject(MaskDC, hMaskBmp); {Select bitmap in DC}
  hInvPrevBmp := SelectObject(invDC, hInvBmp); {Select bitmap in DC}
  hDestPrevBmp := SelectObject(resultDC, hResultBmp); {Select bitmap}

  {Make backup of source bitmap to restore later}
  bSuccess := BitBlt(SaveDC, 0, 0, bmp.bmWidth, bmp.bmHeight, SrcDC, 0, 0, SRCCOPY);
  
  {Create mask: set background color of source to transparent color.}
  OrigColor := SetBkColor(SrcDC, TransColor);
  bSuccess := BitBlt(MaskDC, 0, 0, bmp.bmWidth, bmp.bmHeight, SrcDC, 0, 0, SRCCOPY);
  TransColor := SetBkColor(SrcDC, OrigColor);

  {Create inverse of mask to AND w/ source & combine w/ background.}
  bSuccess := BitBlt(invDC, 0, 0, bmp.bmWidth, bmp.bmHeight, MaskDC, 0, 0, NOTSRCCOPY);
  
  {Copy background bitmap to result & create final transparent bitmap}
  bSuccess := BitBlt(resultDC, 0, 0, bmp.bmWidth, bmp.bmHeight,
    DestDC, DestX, DestY, SRCCOPY);
  
  {AND mask bitmap w/ result DC to punch hole in the background by
    painting black area for non-transparent portion of source bitmap.}
  bSuccess := BitBlt(resultDC, 0, 0, bmp.bmWidth, bmp.bmHeight, MaskDC, 0, 0, SRCAND);
  
  {AND inverse mask w/ source bitmap to turn off bits associated
    with transparent area of source bitmap by making it black.}
  bSuccess := BitBlt(SrcDC, 0, 0, bmp.bmWidth, bmp.bmHeight, invDC, 0, 0, SRCAND);
  
  {XOR result w/ source bitmap to make background show through.}
  bSuccess := BitBlt(resultDC, 0, 0, bmp.bmWidth, bmp.bmHeight, SrcDC, 0, 0, SRCPAINT);

  {Display transparent bitmap on backgrnd}
  bSuccess := BitBlt(DestDC, DestX, DestY, bmp.bmWidth, bmp.bmHeight, resultDC, 0, 0, SRCCOPY);

  {Restore backup of bitmap.}
  bSuccess := BitBlt(SrcDC, 0, 0, bmp.bmWidth, bmp.bmHeight, SaveDC, 0, 0, SRCCOPY);

  hPrevBmp := SelectObject(SrcDC, hSrcPrevBmp); {Select orig object}
  hPrevBmp := SelectObject(SaveDC, hSavePrevBmp); {Select orig object}
  hPrevBmp := SelectObject(resultDC, hDestPrevBmp); {Select orig object}
  hPrevBmp := SelectObject(MaskDC, hMaskPrevBmp); {Select orig object}
  hPrevBmp := SelectObject(invDC, hInvPrevBmp); {Select orig object}

  bSuccess := DeleteObject(hSaveBmp); {Deallocate system resources.}
  bSuccess := DeleteObject(hMaskBmp); {Deallocate system resources.}
  bSuccess := DeleteObject(hInvBmp); {Deallocate system resources.}
  bSuccess := DeleteObject(hResultBmp); {Deallocate system resources.}

  bSuccess := DeleteDC(SrcDC); {Deallocate system resources.}
  bSuccess := DeleteDC(SaveDC); {Deallocate system resources.}
  bSuccess := DeleteDC(invDC); {Deallocate system resources.}
  bSuccess := DeleteDC(MaskDC); {Deallocate system resources.}
  bSuccess := DeleteDC(resultDC); {Deallocate system resources.}
end;

initialization
finalization
end.

