unit mapunit;

interface

uses Windows, SysUtils, Graphics, Classes, tileunit, util;

type
  // Ϯ}H
  PBreakMap = ^TBreakMap;
  TBreakMap = array[0..3, 0..3] of Boolean; // 4 x 4 = 16 ӤpϮ, "}"or"S}"
  
  // aϹϮ
  TCell = class
  private
    FTileNo: Integer; // ϮҩmϤs
    FLayer, FX, fy: Integer; // ϼh, y
    
    FBreakPtr: PBreakMap; // Ϯ}H
    FRect: TRect; // ϮҦϰ, |HϮ}Hܧ
    
    function GetTileAttr: TTileAttr;
    procedure SetTileAttr(const Value: TTileAttr);
    
    function GetTileBitmap: TBitmap; // oϤ bitmap
    
    function GetCanPass: Boolean; // oӹϮ_qL ?
  protected
  public
    constructor Create(Layer, X, Y: Integer);
    destructor Destroy; override;
    
    // Jxs
    procedure LoadFromStream(Stream: TStream);
    procedure SaveToStream(Stream: TStream);
    
    // spϮҦϰ
    procedure CalcRect;
    
    // Ϯ}HBzk
    procedure AllocBreakMap; // إ߹Ϯ}H
    procedure DisposeBreakMap; // Ϯ}H
    procedure SetBreakMap(X, Y: Integer); // ]wϮ}H
    procedure BreakBy(ARect: TRect); // ̯xΰϰ]wϮ}H
    
    property TileNo: Integer read FTileNo write FTileNo;
    property TileAttr: TTileAttr read GetTileAttr write SetTileAttr;
    property TileBitmap: TBitmap read GetTileBitmap;
    
    property Rect: TRect read FRect;
    
    property CanPass: Boolean read GetCanPass;
  end;
  
  // a
  TMapArray = array[0..TILE_NUM_Y - 1, 0..TILE_NUM_X - 1] of TCell;
  
  tmap = class
  private
    FMaps: array[0..LAYER_MAX] of TMapArray; // (LAYER_MAX + 1) ha
    
    FLevelNo: Integer; // ثeJds
    
    FRole_X, FRole_Y: Integer; // ⪺_lm
    
    FBreakBitmap: TBitmap; // Ω}HϮ檺Kϰʧ@
    
    procedure SetLevelNo(const Value: Integer);
    function GetCell(Layer, X, Y: Integer): TCell;
    
    procedure SetRole_X(const Value: Integer);
    procedure SetRole_Y(const Value: Integer);
  protected
  public
    constructor Create;
    destructor Destroy; override;
    
    function GetFileName: string; // ھds, Ǧ^ɦW
    
    procedure LoadFromFile;
    procedure SaveToFile;
    
    // NYaϼheb Canvas W
    procedure DrawMap(Layer: Integer; Canvas: TCanvas);
    
    // ]ia, Υu]Y@h
    procedure ResetAllMap;
    procedure ResetMap(Layer: Integer);
    
    property LevelNo: Integer read FLevelNo write SetLevelNo;
    
    // oY檺aΤΪ~Ϥs, oOw]ݩ
    property Cell[Layer, X, Y: Integer]: TCell read GetCell; default;
    
    // olm
    property Role_X: Integer read FRole_X write SetRole_X;
    property Role_Y: Integer read FRole_Y write SetRole_Y;
  end;
  
var
  Map: tmap;
  
implementation

uses xFiles;

constructor TCell.Create(Layer, X, Y: Integer);
begin
  inherited Create;
  
  FLayer := Layer;
  FX := X;
  fy := Y;
  
  FBreakPtr := nil;
  
  CalcRect; // pҦΰϰ
end;

destructor TCell.Destroy;
begin
  DisposeBreakMap;
  
  inherited Destroy;
end;

// ھڮyФί}HpҦΰϰ
procedure TCell.CalcRect;
  label // goto ҨϥΪ, Ouch !!
  OutLoop1, OutLoop2, OutLoop3, OutLoop4;
var
  X, Y: Integer;
begin
  if not Assigned(FBreakPtr) then // YS}H, hΰϰܦn
    FRect := Classes.Rect(TILEWIDTH[FX], TILEHeight[fy], TILEWIDTH[FX + 1], TILEHeight[fy + 1])
  else
  begin
    // Y}H, hΰϰnwWUkOo
    FRect := Classes.Rect(0, 0, 0, 0);
    
    // ̥
    for X := 0 to 3 do
      for Y := 0 to 3 do
        if not FBreakPtr^[Y, X] then
        begin
          FRect.Left := TILEWIDTH[FX] + X * SM_TILE_WIDTH;
          goto OutLoop1;
        end;
    
    OutLoop1:
      
      // ̥k
      for X := 3 downto 0 do
        for Y := 0 to 3 do
          if not FBreakPtr^[Y, X] then
          begin
            FRect.Right := TILEWIDTH[FX] + (X + 1) * SM_TILE_WIDTH;
            goto OutLoop2;
          end;
    
    OutLoop2:
      
      // ̤W
      for Y := 0 to 3 do
        for X := 0 to 3 do
          if not FBreakPtr^[Y, X] then
          begin
            FRect.Top := TILEHeight[fy] + Y * SM_TILE_HEIGHT;
            goto OutLoop3;
          end;
    
    OutLoop3:
      
      // ̤U
      for Y := 3 downto 0 do
        for X := 0 to 3 do
          if not FBreakPtr^[Y, X] then
          begin
            FRect.Bottom := TILEHeight[fy] + (Y + 1) * SM_TILE_HEIGHT;
            goto OutLoop4;
          end;
    
    OutLoop4:
  end;
end;

// qƬyŪϮ
procedure TCell.LoadFromStream(Stream: TStream);
var
  bHasBreakPtr: Boolean;
begin
  with Stream do
  begin
    read (FTileNo, sizeof(Integer));
    
    read (bHasBreakPtr, sizeof(Boolean));
    if bHasBreakPtr then // Y}H, hŪ
    begin
      AllocBreakMap;
      read (FBreakPtr^, sizeof(TBreakMap));
    end else DisposeBreakMap;
    
    CalcRect; // spҦΰϰ
  end;
end;

// NϮgJƬy
procedure TCell.SaveToStream(Stream: TStream);
var
  bHasBreakPtr: Boolean;
begin
  with Stream do
  begin
    write (FTileNo, sizeof(Integer));
    
    bHasBreakPtr := Assigned(FBreakPtr);
    write (bHasBreakPtr, sizeof(Boolean));
    if bHasBreakPtr then // Y}H, hgJ
      write (FBreakPtr^, sizeof(TBreakMap));
  end;
end;

function TCell.GetTileAttr: TTileAttr;
begin
  Result := Tiles[FTileNo].Attr;
end;

procedure TCell.SetTileAttr(const Value: TTileAttr);
begin
  Tiles[FTileNo].Attr := Value;
end;

// omϮ檺Ϥ
function TCell.GetTileBitmap: TBitmap;
begin
  Result := Tiles[FTileNo].Bitmap;
end;

// إ߹Ϯ}H
procedure TCell.AllocBreakMap;
begin
  if not Assigned(FBreakPtr) then
    New(FBreakPtr);
  FillChar(FBreakPtr^, sizeof(TBreakMap), 0);
end;

// Ϯ}H
procedure TCell.DisposeBreakMap;
begin
  if Assigned(FBreakPtr) then
  begin
    Dispose(FBreakPtr);
    FBreakPtr := nil;
  end;
end;

// ]wϮ}H
procedure TCell.SetBreakMap(X, Y: Integer);
begin
  if not Assigned(FBreakPtr) then
    AllocBreakMap;
  
  FBreakPtr^[Y, X] := True; // (X, Y) }F
  
  CalcRect; // spҦΰϰ
end;

// ̯xΰϰ]wϮ}H
procedure TCell.BreakBy(ARect: TRect);
var
  X, Y : Integer;
  R, R1: TRect;  
begin
  if not Assigned(FBreakPtr) then // Y٨S}H, Nؤ@
    AllocBreakMap;
  
  // P ARect 涰, 涰ph]}
  for Y := 0 to 3 do
    for X := 0 to 3 do
    begin
      // pp檺xΰϰ
      R1 := Classes.Rect(TILEWIDTH[FX] + X * SM_TILE_WIDTH, TILEHeight[fy] + Y * SM_TILE_HEIGHT,
        TILEWIDTH[FX] + (X + 2) * SM_TILE_WIDTH, TILEHeight[fy] + (Y + 2) * SM_TILE_HEIGHT);
      
      // Y涰, h}
      if IntersectRect(R, ARect, R1) then
        FBreakPtr^[Y, X] := True;
    end;
  
  CalcRect; // spҦΰϰ
end;

// ϮO_qL
function TCell.GetCanPass: Boolean;
begin
  Result := (TileNo = 0) or (tacanpass in TileAttr);
end;

{ TMap class }

constructor tmap.Create;
var
  L, X, Y: Integer;
begin
  inherited Create;
  
  FLevelNo := 0;
  
  // إߨC@hҦϮ
  for L := 0 to LAYER_MAX do
    for Y := 0 to TILE_NUM_Y - 1 do
      for X := 0 to TILE_NUM_X - 1 do
        FMaps[L][Y, X] := TCell.Create(L, X, Y);
  
  // Ω}HϮKϰʧ@ bitmap
  FBreakBitmap := TBitmap.Create;
  FBreakBitmap.Width := SM_TILE_WIDTH;
  FBreakBitmap.Height := SM_TILE_HEIGHT;
  FBreakBitmap.Transparent := True;
  FBreakBitmap.TransparentColor := TRANSPARENT_COLOR;
end;

destructor tmap.Destroy;
var
  L, X, Y: Integer;
begin
  for L := 0 to LAYER_MAX do
    for Y := 0 to TILE_NUM_Y - 1 do
      for X := 0 to TILE_NUM_X - 1 do
        FMaps[L][Y, X].Free;
  
  FBreakBitmap.Free;
  
  inherited Destroy;
end;

function tmap.GetFileName: string;
begin
  Result := Format('%s%s%d%s',[AppDir, FN_MAP_PREFIX, FLevelNo, FN_MAP_EXT]);
end;

// qɮŪa
procedure tmap.LoadFromFile;
var
  fs     : TFileStream;
  I, X, Y: Integer;    
begin
  fs := TFileStream.Create(GetFileName, fmOpenRead);
  with fs do
    try
      CheckSignature(fs, SIG_MYFILE); // ˬdY
      CheckSignature(fs, ClassName); // ˬdY
      
      // Ūlm
      read (FRole_X, sizeof(Integer));
      read (FRole_Y, sizeof(Integer));
      
      // ŪC@hҦϮ
      for I := 0 to LAYER_MAX do
        for Y := 0 to TILE_NUM_Y - 1 do
          for X := 0 to TILE_NUM_X - 1 do
            FMaps[I][Y, X].LoadFromStream(fs);
    finally
      Free;
    end;
end;

// NaϼgJɮ
procedure tmap.SaveToFile;
var
  fs     : TFileStream;
  S      : string;     
  I, X, Y: Integer;    
begin
  fs := TFileStream.Create(GetFileName, fmCreate or fmOpenWrite);
  with fs do
    try
      write (SIG_MYFILE, Length(SIG_MYFILE));
      S := ClassName;
      write (S[1], Length(S));
      
      // gJlm
      write (FRole_X, sizeof(Integer));
      write (FRole_Y, sizeof(Integer));
      
      // gJC@hҦϮ      
      for I := 0 to LAYER_MAX do
        for Y := 0 to TILE_NUM_Y - 1 do
          for X := 0 to TILE_NUM_X - 1 do
            FMaps[I][Y, X].SaveToStream(fs);
    finally
      Free;
    end;
end;

procedure tmap.SetLevelNo(const Value: Integer);
begin
  FLevelNo := Value;
  LoadFromFile;
end;

// Nwaϼheb Canvas W
procedure tmap.DrawMap(Layer: Integer; Canvas: TCanvas);
  
  procedure DrawBrokenCell(X, Y: Integer);
  var
    M, N: Integer; // øs}HpϮ
  begin
    // Y}HϮ, h@@NsbpϮKW
    for N := 0 to 3 do
      for M := 0 to 3 do
      begin
        // YpϮ}FNεe
        if FMaps[Layer][Y, X].FBreakPtr[N, M] then continue;

        // ƻst@ bitmap
        FBreakBitmap.Canvas.CopyRect(Rect(0, 0, SM_TILE_WIDTH,
          SM_TILE_HEIGHT), FMaps[Layer][Y, X].TileBitmap.Canvas,
          Rect(M * SM_TILE_WIDTH, N * SM_TILE_HEIGHT,
          (M + 1) * SM_TILE_WIDTH, (N + 1) * SM_TILE_HEIGHT));

        // AKeW, HFzKϮĪG
        Canvas.Draw(TILEWIDTH[X] + M * SM_TILE_WIDTH,
          TILEHeight[Y] + N * SM_TILE_HEIGHT, FBreakBitmap);
      end;
  end;

var
  X, Y: Integer; // øsϮ
begin
  if Layer = LAYER_TERR then // Yaμh, hK
    for Y := 0 to TILE_NUM_Y - 1 do // C@Ϯ
      for X := 0 to TILE_NUM_X - 1 do
        BitBlt(Canvas.Handle, TILEWIDTH[X], TILEHeight[Y],
        TILE_WIDTH, TILE_Height, FMaps[Layer][Y, X].TileBitmap.Canvas.Handle, 0, 0, SRCCOPY)
  else
  begin
    // 䥦aϼh, hĥγzK
    for Y := 0 to TILE_NUM_Y - 1 do // C@Ϯ
      for X := 0 to TILE_NUM_X - 1 do
      begin
        // s 0 ܨSϤ
        if FMaps[Layer][Y, X].TileNo = 0 then continue;

        // YϮO㪺
        if not Assigned(FMaps[Layer][Y, X].FBreakPtr) then
          Canvas.Draw(TILEWIDTH[X], TILEHeight[Y],
          FMaps[Layer][Y, X].TileBitmap)
        else
          DrawBrokenCell(X, Y); // øX}HϮ
      end;
  end;
end;

// ]a
procedure tmap.ResetAllMap;
var
  I: Integer;
begin
  for I := 0 to LAYER_MAX do
    ResetMap(I);
  
  FRole_X := 0;
  FRole_Y := 0;
end;

// ]waϼh
procedure tmap.ResetMap(Layer: Integer);
var
  X, Y: Integer;
begin
  for Y := 0 to TILE_NUM_Y - 1 do
    for X := 0 to TILE_NUM_X - 1 do
      FMaps[Layer][Y, X].TileNo := 0;
end;

function tmap.GetCell(Layer, X, Y: Integer): TCell;
begin
  Result := FMaps[Layer][Y, X];
end;

procedure tmap.SetRole_X(const Value: Integer);
begin
  FRole_X := Value;
end;

procedure tmap.SetRole_Y(const Value: Integer);
begin
  FRole_Y := Value;
end;

end.

