unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, StdCtrls, ExtCtrls, Spin, Grids, ExtDlgs, Math, TileUnit, ComCtrls;

type
  TMainForm = class(TForm)
    mnuMain: TMainMenu;
    F1: TMenuItem;
    H1: TMenuItem;
    mnuAbout: TMenuItem;
    mnuExit: TMenuItem;
    N1: TMenuItem;
    mnuOpen: TMenuItem;
    mnuSave: TMenuItem;
    mnuNew: TMenuItem;
    dlgOpen: TOpenDialog;
    dlgSave: TSaveDialog;
    mnuSaveAs: TMenuItem;
    N3: TMenuItem;
    dlgOpenPicture: TOpenPictureDialog;
    tvwTiles: TTreeView;
    stbMain: TStatusBar;
    pnlRight: TPanel;
    Splitter1: TSplitter;
    grdTile: TDrawGrid;
    lblTileNo: TLabel;
    lblDimension: TLabel;
    GroupBox1: TGroupBox;
    cbxCanPass: TCheckBox;
    cbxBulletCanPass: TCheckBox;
    cbxCanBreak: TCheckBox;
    mnuTiles: TMenuItem;
    mnuAdd: TMenuItem;
    mnuRemove: TMenuItem;
    cbxFlag: TCheckBox;
    procedure mnuExitClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure mnuNewClick(Sender: TObject);
    procedure mnuOpenClick(Sender: TObject);
    procedure mnuSaveClick(Sender: TObject);
    procedure grdTileDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure mnuAboutClick(Sender: TObject);
    procedure grdTileSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure tvwTilesChange(Sender: TObject; Node: TTreeNode);
    procedure mnuAddClick(Sender: TObject);
    procedure mnuRemoveClick(Sender: TObject);
  private
    FFileName: string;
    FFirstNo: Integer;
    
    procedure BackupAttr(Col, Row: Integer);
    procedure RestoreAttr(Col, Row: Integer);
    
    procedure UpdateControlStatus;
  public
    { Public declarations }
  end;
  
var
  MainForm: TMainForm;
  
implementation

uses util, about, xFiles;

{$R *.DFM}

procedure TMainForm.mnuExitClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Tiles := TTiles.Create; // إ߹Ϯw

  mnuNewClick(nil);
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Tiles.Free;
end;

procedure TMainForm.mnuNewClick(Sender: TObject);
begin
  Tiles.FreeTiles; // sϮw
  FFileName := '';
  FFirstNo := -1;
  tvwTiles.Items.Clear;

  UpdateControlStatus;
end;

procedure TMainForm.mnuOpenClick(Sender: TObject);
begin
  with dlgOpen do
    if Execute then // }ɹܲ
      try
        mnuNewClick(nil); // Ϯwe

        // JϮw
        Tiles.LoadFromFile(Filename);
        // ŪϤsմyz
        ReadComponentResFile(ChangeFileExt(Filename, '.TVW'),
          tvwTiles);

        FFileName := Filename; // ŪaιϮw\

       // Yܥs, hLܲĤ@Ӹ`I        
        if tvwTiles.Selected = nil then
          tvwTiles.Selected := tvwTiles.Items[0];
      finally
        UpdateControlStatus;
      end;
end;

procedure TMainForm.mnuSaveClick(Sender: TObject);
begin
  // YO"tss" ٥wɦW, NݨϥΪɦW
  if ((Sender as TComponent).Tag = 1) or (FFileName = '') then
  begin
    dlgSave.Filter := dlgOpen.Filter;
    // ߰ݨϥΪɦW, YN}
    if not dlgSave.Execute then Exit;
    FFileName := dlgSave.Filename; // NɦWO_
  end;

  BackupAttr(grdTile.Col, grdTile.Row); // xsثeϤݩ

  Tiles.SaveToFile(FFileName); // xsϮw
  // xsϤsմyz
  WriteComponentResFile(ChangeFileExt(FFileName, '.TVW'),
    tvwTiles);
  UpdateControlStatus; // sD
end;

procedure TMainForm.UpdateControlStatus;
var
  sFileName: string;
begin
  sFileName := FFileName;
  if sFileName = '' then sFileName := 'Untitled';
  Caption := Format('%s - %s',[Application.Title, sFileName]);
  
  stbMain.SimpleText := Format('Ϥ`: %d',[Tiles.TileNum]);
  
  mnuSave.Enabled := Tiles.TileNum <> 0;
  mnuSaveAs.Enabled := Tiles.TileNum <> 0;
  
  grdTile.Invalidate;
end;

procedure TMainForm.grdTileDrawCell(Sender: TObject; ACol,
    ARow: Integer; Rect: TRect; State: TGridDrawState);
var
  No: Integer; // Ϥs
begin
  if tvwTiles.Selected = nil then Exit; // @wnܬYsդ~

  // ҹϤs = iϤs + ARow * bƥ + ACol
  No := Integer(tvwTiles.Selected.Data) +
    ARow * grdTile.ColCount + ACol;
  if No >= Tiles.TileNum then Exit; // O_Xks ?

  with grdTile.Canvas do // eXϤ
    Draw(Rect.Left, Rect.Top, Tiles[No].Bitmap);
end;

procedure TMainForm.grdTileSelectCell(Sender: TObject; ACol,
    ARow: Integer; var CanSelect: Boolean);
begin
  BackupAttr(grdTile.Col, grdTile.Row);
  RestoreAttr(ACol, ARow);

  lblTileNo.Caption := Format('s: %d (%d - %d)',[FFirstNo, FFirstNo, FFirstNo + Tiles[FFirstNo].xNum * Tiles[FFirstNo].yNum - 1]);
end;

procedure TMainForm.BackupAttr(Col, Row: Integer);
var
  No: Integer;
begin
  if FFirstNo = -1 then Exit;

  No := FFirstNo + Row * grdTile.ColCount + Col; // p쥻ܪϤs

  // NϥΪ̳]wϤݩʼg^h
  Tiles[No].Attr := [];

  // iHVa
  if cbxCanPass.Checked then Tiles[No].Attr := Tiles[No].Attr + [taCanpass];
  if cbxBulletCanPass.Checked then Tiles[No].Attr := Tiles[No].Attr + [taBulletCanpass];
  if cbxCanBreak.Checked then Tiles[No].Attr := Tiles[No].Attr + [taCanBreak];
  if cbxFlag.Checked then Tiles[No].Attr := Tiles[No].Attr + [taFlag];
end;

procedure TMainForm.RestoreAttr(Col, Row: Integer);
var
  No: Integer;
begin
  if FFirstNo = -1 then Exit;

  No := FFirstNo + Row * grdTile.ColCount + Col; // pYNܪϤs

  // NҿܪϤݩʥ checkbox {X
  cbxCanPass.Checked := taCanpass in Tiles[No].Attr;
  cbxBulletCanPass.Checked := taBulletCanpass in Tiles[No].Attr;
  cbxCanBreak.Checked := taCanBreak in Tiles[No].Attr;
  cbxFlag.Checked := taFlag in Tiles[No].Attr;
end;

procedure TMainForm.mnuAboutClick(Sender: TObject);
begin
  with TAboutBox.Create(self) do
    try
      ShowModal;
    finally
      Free;
    end;
end;

procedure TMainForm.tvwTilesChange(Sender: TObject; Node: TTreeNode);
var
  No: Integer;
begin
  if tvwTiles.Selected = nil then Exit;

  // x^ثe
  BackupAttr(grdTile.Col, grdTile.Row);

  No := Integer(tvwTiles.Selected.Data);
  if No < Tiles.TileNum then
  begin
    grdTile.ColCount := Tiles[No].xNum;
    grdTile.RowCount := Tiles[No].yNum;

    grdTile.Row := 0;
    grdTile.Col := 0;

    FFirstNo := No;
    lblDimension.Caption := Format('jp: %d x %d',[Tiles[FFirstNo].xNum, Tiles[FFirstNo].yNum]);

    RestoreAttr(0, 0);
  end;

  UpdateControlStatus;
end;

procedure TMainForm.mnuAddClick(Sender: TObject);

  procedure AddTreeNode(const Filename: string; FirstNo: Integer);
  begin
    // b˵󤤥[JϤsժ`I]yz^
    // Ϥsմyzw]Ȭ[J BMP ϧɦW
    with tvwTiles.Items.Add(nil,
      ExtractFileNameNoExt(ExtractFileName(Filename))) do
    begin
      Data := Pointer(FirstNo);  // OsչĤ@iϤs
    end;

    // Yܥs, hLܲĤ@Ӹ`I
    if tvwTiles.Selected = nil then
      tvwTiles.Selected := tvwTiles.Items[0];
  end;

var
  Bits         : TBitmap; // [J bitmap
  X, Y, FirstNo: Integer;
  xNum, yNum   : Integer; // ϧΤΫ᪺babϤƥ
  Tile         : TTile;   // ͪsϤ
begin
  if dlgOpenPicture.Execute then
    try
      Bits := TBitmap.Create; // ͤθJ[JϮw bitmap
      Bits.LoadFromFile(dlgOpenPicture.Filename);

      // Y bitmap iϤؤo٤p, LkBz
      if (Bits.Width < TILE_WIDTH) or (Bits.Height < TILE_HEIGHT) then
        raise Exception.Create('Bitmap is too small');

      // ̤jO 5 x 5 Ϥs
      xNum := Min(5, Bits.Width div TILE_WIDTH);
      yNum := Min(5, Bits.Height div TILE_HEIGHT);

      // ̧ǤΥX xNum * yNum ӹϤ
      for Y := 0 to yNum - 1 do
        for X := 0 to xNum - 1 do
        begin
          Tile := TTile.Create;  // ͹Ϥ
          // NϧνƻsϤ bitmap W
          Tile.Bitmap.Canvas.CopyRect(TILE_RECT, Bits.Canvas,
            Rect(X * TILE_WIDTH, Y * TILE_HEIGHT,
            (X + 1) * TILE_WIDTH, (Y + 1) * TILE_HEIGHT));

          Tile.FirstTile := (X = 0) and (Y = 0); // OOsYY
          Tile.xNum := xNum;  // OsYY, tdO
          Tile.yNum := yNum;  // ϤsժbabϤƥ

          // NͪsϤ[JϮw
          if Tile.FirstTile then
            FirstNo := Tiles.AddTile(Tile) // osժYYs
          else
            Tiles.AddTile(Tile);
        end;

      // Nsմyz[J˵
      AddTreeNode(dlgOpenPicture.Filename, FirstNo);

      UpdateControlStatus;
    finally
      Bits.Free;  // lvSΤF, 
    end;
end;

procedure TMainForm.mnuRemoveClick(Sender: TObject);
var
  I, No: Integer;
begin
  if tvwTiles.Selected = nil then Exit; // @wnwYӸsդ~

  // oثeϤsխiϤs
  No := Integer(tvwTiles.Selected.Data);

  // NӹϤsժϤ]"m"
  for I := 0 to Tiles[No].xNum * Tiles[No].yNum - 1 do
    Tiles[No + I].Disposed := True;

  tvwTiles.Selected.Delete; // NϤsմyz屼
  UpdateControlStatus;
end;

end.


