unit xWindows;

interface

uses Windows, Sysutils, Controls, Forms;

const
  DEFAULT_FOCUSRECT_WIDTH = 3;
  
function FindControl(Handle: HWND; ProcessID: DWORD = 0): TWinControl;
function IsDelphiHandle(Handle: HWND; ProcessID: DWORD = 0): Boolean;

function FindTopmostWindow(ParentWnd: HWND; Pt: TPoint): HWND;

procedure MyDrawFocusRect(DC: HDC; R: TRect; Width: Integer = DEFAULT_FOCUSRECT_WIDTH);
procedure DrawWindowFocusRect(Wnd: HWND; Width: Integer = DEFAULT_FOCUSRECT_WIDTH);
procedure DrawControlFocusRect(Control: TControl; Width: Integer = DEFAULT_FOCUSRECT_WIDTH);

implementation

uses xKernel;

var
  WindowAtom : TAtom;               
  ControlAtom: TAtom;               
  
  AtomText   : array[0..31] of Char;
  
  { Find a TWinControl given a window handle }
  
function FindControl(Handle: HWND; ProcessID: DWORD = 0): TWinControl;
begin
//  DebugStrFmt('Find Control Handle = %x',[Handle]);
  if Handle <> 0 then
  begin
    if ProcessID = 0 then
      GetWindowThreadProcessId(Handle, @ProcessID);
    WindowAtom := GlobalAddAtom(StrFmt(AtomText, 'Delphi%.8X',[ProcessID]));
    Result := Pointer(GetProp(Handle, MakeIntAtom(WindowAtom)));
//    DebugStrFmt('Find Control PID = %x Result = %p',[ProcessID, Pointer(Result)]);
  end else
    Result := nil;
end;

function IsDelphiHandle(Handle: HWND; ProcessID: DWORD = 0): Boolean;
begin
  Result := FindControl(Handle, ProcessID) <> nil;
end;

function FindTopmostWindow(ParentWnd: HWND; Pt: TPoint): HWND;
var
  R: TRect;
begin
  Result := GetWindow(ParentWnd, GW_CHILD); // oĤ@Ӥl

  while Result <> 0 do
  begin
    GetWindowRect(Result, R); // oxΰϰ

    if IsWindowVisible(Result) and PtInRect(R, Pt) then
    begin
      if Result = Application.MainForm.Handle then // { Main Form
      begin
        Result := 0;
        Exit;
      end;
      Result := FindTopmostWindow(Result, Pt); // , jIsol
      Exit;
    end;
    Result := GetWindow(Result, GW_HWNDNEXT); // oU@ siblings 
  end;

  Result := ParentWnd; // 䤣, NOڭ̭n䪺
end;

{
  ϥ ChildWindowFromPoint ӴM, s non-visual |

function FindTopmostWindow(ParentWnd: HWND; Pt: TPoint): HWND;
begin
  repeat
    Result := ChildWindowFromPoint(ParentWnd, Pt);
    if Result = ParentWnd then break;

    if Result = Application.MainForm.Handle then // YO MainForm , Ǧ^ 0
    begin
      Result := 0;
      Exit;
    end;

    MapWindowPoints(ParentWnd, Result, Pt, 1);
    ParentWnd := Result;
  until false;
end;
}
procedure MyDrawFocusRect(DC: HDC; R: TRect; Width: Integer = DEFAULT_FOCUSRECT_WIDTH);
var
  I: Integer;
begin
  InflateRect(R, - Width, - Width);
  DrawFocusRect(DC, R);
  for I := 1 to Width do
  begin
    InflateRect(R, 1, 1);
    DrawFocusRect(DC, R);
  end;
end;

procedure DrawWindowFocusRect(Wnd: HWND; Width: Integer = DEFAULT_FOCUSRECT_WIDTH);
var
  DC: HDC;  
  R : TRect;
begin
  DC := getwindowdc(Wnd);
  GetWindowRect(Wnd, R);
  MapWindowPoints(HWND_DESKTOP, Wnd, R, 2);
  OffsetRect(R, - R.left, - R.top);
  MyDrawFocusRect(DC, R, Width);
  releasedc(Wnd, DC);
end;

procedure DrawControlFocusRect(Control: TControl; Width: Integer = DEFAULT_FOCUSRECT_WIDTH);
var
  Wnd: HWND; 
  DC : HDC;  
  R  : TRect;
begin
  Wnd := Control.Parent.Handle;
  DC := getdc(Wnd);
  R := Control.BoundsRect;
  MyDrawFocusRect(DC, R, Width);
  releasedc(Wnd, DC);
end;


end.

